"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SchemaAnalyzer = void 0;
const lodash_1 = __importDefault(require("lodash"));
const reservoir_1 = __importDefault(require("reservoir"));
const semantic_types_1 = __importDefault(require("./semantic-types"));
function extractStringValueFromBSON(value) {
    if (value && value._bsontype) {
        if (['Decimal128', 'Long'].includes(value._bsontype)) {
            return value.toString();
        }
        if (['Double', 'Int32'].includes(value._bsontype)) {
            return String(value.value);
        }
    }
    if (typeof value === 'string') {
        return value;
    }
    return String(value);
}
function fieldComparator(a, b) {
    const aName = a.name;
    const bName = b.name;
    if (aName === '_id') {
        return -1;
    }
    if (bName === '_id') {
        return 1;
    }
    return aName.toLowerCase() < bName.toLowerCase() ? -1 : 1;
}
function finalizeSchema(schema, parent, tag) {
    if (schema === undefined) {
        return;
    }
    if (tag === undefined) {
        finalizeSchema(schema.fields, schema, 'fields');
    }
    if (tag === 'fields') {
        Object.values(schema).forEach((field) => {
            const missing = parent.count - field.count;
            if (missing > 0) {
                field.types.Undefined = {
                    name: 'Undefined',
                    type: 'Undefined',
                    path: field.path,
                    count: missing
                };
            }
            field.total_count = Object.values(field.types)
                .map((v) => v.count)
                .reduce((p, c) => p + c, 0);
            finalizeSchema(field.types, field, 'types');
            field.type = field.types.map((v) => v.name);
            if (field.type.length === 1) {
                field.type = field.type[0];
            }
            field.has_duplicates = !!field.types.find((v) => v.has_duplicates);
            field.probability = field.count / parent.count;
        });
        parent.fields = Object.values(parent.fields).sort(fieldComparator);
    }
    if (tag === 'types') {
        Object.values(schema).forEach((type) => {
            type.total_count = (type.lengths || []).reduce((p, c) => p + c || 0, 0);
            finalizeSchema(type.fields, type, 'fields');
            finalizeSchema(type.types, type, 'types');
            type.probability = type.count / (parent.total_count || parent.count);
            if (type.name === 'Null' || type.name === 'Undefined') {
                delete type.values;
                type.unique = type.count === 0 ? 0 : 1;
                type.has_duplicates = type.count > 1;
            }
            else if (type.values) {
                type.unique = new Set(type.values.map(extractStringValueFromBSON)).size;
                type.has_duplicates = type.unique !== type.values.length;
            }
            if (type.lengths) {
                type.average_length = type.total_count / type.lengths.length;
            }
        });
        parent.types = Object.values(parent.types).sort((a, b) => b.probability - a.probability);
    }
}
function getBSONType(value) {
    let T;
    if (value && value._bsontype) {
        T = value._bsontype;
    }
    else {
        T = Object.prototype.toString.call(value).replace(/^\[object (\w+)\]$/, '$1');
    }
    if (T === 'Object') {
        T = 'Document';
    }
    return T;
}
function addToValue(type, value) {
    if (type.name === 'String') {
        if (value.length > 10000) {
            value = value.charCodeAt(10000 - 1) === value.codePointAt(10000 - 1)
                ? value.slice(0, 10000)
                : value.slice(0, 10000 - 1);
        }
    }
    type.values.pushSome(value);
}
class SchemaAnalyzer {
    constructor(options) {
        this.options = { semanticTypes: false, storeValues: true, ...options };
        this.finalized = false;
        this.semanticTypes = {
            ...semantic_types_1.default
        };
        if (typeof this.options.semanticTypes === 'object') {
            const enabledTypes = Object.entries(this.options.semanticTypes)
                .filter(([, v]) => typeof v === 'boolean' && v)
                .map(([k]) => k.toLowerCase());
            this.semanticTypes = {
                ...Object.entries(this.semanticTypes)
                    .filter(([k]) => enabledTypes.includes(k.toLowerCase()))
                    .reduce((p, [k, v]) => ({ ...p, [k]: v }), {})
            };
            Object.entries(this.options.semanticTypes)
                .filter(([, v]) => typeof v === 'function')
                .forEach(([k, v]) => {
                this.semanticTypes[k] = v;
            });
        }
        this.rootSchema = {
            fields: {},
            count: 0
        };
    }
    getSemanticType(value, path) {
        const returnValue = Object.entries(this.semanticTypes)
            .filter(([, v]) => {
            return v(value, path);
        })
            .map(([k]) => k)[0];
        return returnValue;
    }
    addToType(path, value, schema) {
        var _a;
        const bsonType = getBSONType(value);
        const typeName = (this.options.semanticTypes) ? this.getSemanticType(value, path) || bsonType : bsonType;
        const type = schema[typeName] = lodash_1.default.get(schema, typeName, {
            name: typeName,
            bsonType: bsonType,
            path: path,
            count: 0
        });
        type.count++;
        if (typeName === 'Array') {
            type.types = (_a = type.types) !== null && _a !== void 0 ? _a : {};
            type.lengths = type.lengths || [];
            type.lengths.push(value.length);
            value.forEach((v) => this.addToType(path, v, type.types));
        }
        else if (typeName === 'Document') {
            type.fields = lodash_1.default.get(type, 'fields', {});
            Object.entries(value).forEach(([k, v]) => this.addToField(`${path}.${k}`, v, type.fields));
        }
        else if (this.options.storeValues) {
            const defaultValue = bsonType === 'String'
                ? (0, reservoir_1.default)(100) : (0, reservoir_1.default)(10000);
            type.values = type.values || defaultValue;
            addToValue(type, value);
        }
    }
    addToField(path, value, schema) {
        const pathSplitOnDot = path.split('.');
        const defaults = {
            [path]: {
                name: pathSplitOnDot[pathSplitOnDot.length - 1],
                path: path,
                count: 0,
                types: {}
            }
        };
        lodash_1.default.defaultsDeep(schema, defaults);
        const field = schema[path];
        field.count++;
        this.addToType(path, value, field.types);
    }
    analyzeDoc(doc) {
        for (const key of Object.keys(doc)) {
            this.addToField(key, doc[key], this.rootSchema.fields);
        }
        this.rootSchema.count += 1;
    }
    getResult() {
        if (!this.finalized) {
            finalizeSchema(this.rootSchema);
            this.finalized = true;
        }
        return this.rootSchema;
    }
}
exports.SchemaAnalyzer = SchemaAnalyzer;
//# sourceMappingURL=schema-analyzer.js.map